<?php
// ========================================================================
// SlideView: Image Slideshow with PHP JavaScript CSS
// Version: 2020-01-08 20:37 (UTC+01:00)
// https://www.dunweber.com/docs/scripts/releases/slideshow
// Copyright (C) Christian L. Dünweber
// This program is licensed under the MIT License:
// https://opensource.org/licenses/MIT
// ------------------------------------------------------------------------
// Usage:
// require_once('SlideView.php');
// $slideView = new SlideView();
// echo $slideView->getSlides('path/dir1');        // First call, gets ID 1
// echo $slideView->getSlides('path/dir2', '', 7); // Next call, force ID 7
// echo $slideView->getSlides(['img1','img2']);    // Specific images only
// Direct link to image: http://example.com/path/?slide=1&img=3
// ========================================================================

class SlideView
{
	// Auto assigned ID for multiple shows if not specifying a positive $forceID
	private $slideID = 1;

	// First call (with multilple shows) to only show slideShow container once
	private $firstCall = true;

	// File extensions considered images
	private const IMGTYPES = ['jpg','jpeg','png','gif','tif','tiff','ico','bmp'];

	// ====================================================================
	// If $dir is a path all contained images are read.
	// If $dir is an array it must contain paths to images (e.g. to show
	// a selection of a larger set or for custom sorting).
	// In either case the images will be shown as previews and the
	// slideshow prepared for showing upon clicking a preview.
	// A custom css class can be specified to modify or add style.
	// If an integer ID is not specified one will be auto-assigned.
	// --------------------------------------------------------------------
	public function getSlides($dir, string $css = '', int $forceID = 0): string
	{
		// $dir can be a path to a folder with images to read
		if (is_string($dir)) {
			if (!$files = scandir($dir, SCANDIR_SORT_ASCENDING)) {
				return '<p>Couldn\'t open: <em>'.$dir.'</em></p>';
			}

			$imgs = $this->getImages($dir, $files);
			$dir .= '/';
		}
		// Alternatively $dir is an array of paths to images
		else if (is_array($dir)) {
			$imgs = $dir;
			$dir = '';
		}
		else {
			return '<p>Unknown input type: <em>'.gettype($dir).'</em></p>';
		}

		// Use given ID else auto-assign
		$this->slideID = $forceID > 0 ? $forceID : $this->slideID;

		$previews = '';
		$slideArea = '';

		for ($i = 0; $i < count($imgs); $i++) {
			$previews .= '<img class="imgSrcs'.$this->slideID.'"'.
				' onclick="slide.openView('.$this->slideID.','.($i+1).')"'.
				' onkeydown="slide.openPressed(event,'.$this->slideID.','.($i+1).')"'.
				' src="'.$dir.$imgs[$i].'" tabindex="0" alt="" />';
		}

		$previews = '<div class="previews'.(strlen($css) > 0 ? ' '.$css : '').'">'.$previews.'</div>';

		// Only show slideShow container for first call as it's reused for all shows
		if ($this->firstCall) {
			$slideArea = $this->getSlideShow();
			$this->firstCall = false;
		}

		$this->slideID++;
		return '<div class="slideView">'.$slideArea.$previews.'</div>';
	}

	// ====================================================================
	// Get the slideshow full screen view container and reference to the
	// JavaScript script running the show. Is initially hidden.
	// --------------------------------------------------------------------
	private function getSlideShow(): string
	{
		$path = $this->getScriptWebPath();
		$path = htmlspecialchars($path, ENT_QUOTES|ENT_SUBSTITUTE|ENT_HTML5, 'UTF-8', true);

		return
		'<script src="'.$path.'/SlideView.js"></script>
		<script>const slide = new SlideView("'.$path.'");</script>
		<div id="slideShow" tabindex="0" style="display:none;">
			<div id="slideArea">
				<div id="slides"></div>
			</div>
			<div class="button" id="slideCount"></div>
			<div class="button" id="slideInfo"></div>
			<div class="button buttonClose" onclick="slide.closeView()">&#10006;</div>
			<div class="button buttonLeft" onclick="slide.changeSlide(-1)">&#10094;</div>
			<div class="button buttonRight" onclick="slide.changeSlide(1)">&#10095;</div>
		</div>'."\n";
	}

	// ====================================================================
	// Filters images in $files and returns their paths in a new array.
	// --------------------------------------------------------------------
	private function getImages(string $dir, array $files): array
	{
		$imgs = [];
		$path = realpath($dir).DIRECTORY_SEPARATOR;

		for ($i = 0; $i < count($files); $i++) {
			$filename = $files[$i];
			$name = pathinfo($filename, PATHINFO_FILENAME);
			$ext = pathinfo($filename, PATHINFO_EXTENSION);

			// Only image files
			if (is_file($path.$filename) && $this->isImage($ext)) {
				// If a _small is provided it's used for preview instead of the big version
				$small = $name.'_small.'.$ext;

				if (is_file($path.$small)) {
					$imgs[] = $small;
					$i++; // Skip next as it in fact is the _small
				}
				else {
					// Read all images even without corresponding _small
					$imgs[] = $filename;
				}
			}
		}

		return $imgs;
	}

	// ====================================================================
	// Check if file extension indicates an image.
	// --------------------------------------------------------------------
	private function isImage(string $ext): bool
	{
		for ($i = 0; $i < count(Self::IMGTYPES); $i++) {
			if (strcasecmp($ext, Self::IMGTYPES[$i]) === 0) {
				return true;
			}
		}

		return false;
	}

	// ====================================================================
	// Get absolute web path to this folder, where the script resides.
	// --------------------------------------------------------------------
	private function getScriptWebPath(): string
	{
		return str_replace('\\', '/', mb_substr(__DIR__, mb_strlen(realpath($_SERVER['DOCUMENT_ROOT']))));
	}
}

?>
